//
// test_sink.hh     : Test Sink App Include File
// author           : Fabio Silva
//
// Copyright (C) 2000-2003 by the University of Southern California
// $Id: test_sink.hh,v 1.1 2005/04/22 06:08:36 fstann Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#ifndef _TEST_SINK_HH_
#define _TEST_SINK_HH_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "test_common.hh"
#include <list>

using std::list;

class EventEntry {
public:
  EventEntry(int source_id, double latency) :
    source_id_(source_id), latency_(latency) {};

  int source_id_;
  double latency_;
};

typedef list<EventEntry *> EventsList;

class TestSinkReceive;

class TestSinkApp : public DiffApp {
public:
  TestSinkApp(int argc, char **argv);
  virtual ~TestSinkApp() {};

  void recv(NRAttrVec *data, NR::handle my_handle);
  void run();
  void stop();

private:
  // NR Specific variables
  TestSinkReceive *mr_;
  handle subHandle_;

  // Gear Receiver App variables
  bool using_push_;
  bool using_gear_;
  int num_msg_recv_;
  int my_id_;
  float lat_min_;
  float lat_max_;
  float long_min_;
  float long_max_;
  float lat_pt_;
  float long_pt_;

  // Experiment-specific variables
  struct timeval start_time_;
  EventsList events_list_;

  handle setupSubscription();
  void parseCommandLine(int argc, char **argv);
  void readGeographicCoordinates();
  void usage(char *s);
};

class TestSinkReceive : public NR::Callback {
public:
  TestSinkReceive(TestSinkApp *app) : app_(app) {};
  void recv(NRAttrVec *data, NR::handle my_handle);

  TestSinkApp *app_;
};

#endif // !_TEST_SINK_HH_
